@@ -323,11 +323,11 @@ add_themes_from_dir (DialogData *data,
 		}
 
 		file = g_file_get_child (dir, g_file_info_get_name (file_info));
-		if (! g_load_file_in_buffer (file,
-					     (void **) &buffer,
-					     &size,
-					     NULL,
-					     NULL))
+		if (! _g_file_load_in_buffer (file,
+					      (void **) &buffer,
+					      &size,
+					      NULL,
+					      NULL))
 		{
 			g_object_unref (file);
 			g_object_unref (file_info);
@@ -368,7 +368,6 @@ add_themes_from_dir (DialogData *data,
 static void
 load_themes (DialogData *data)
 {
-	char         *style_path;
 	GFile        *style_dir;
 	int           visible_columns;
 	int           col_spacing;
@@ -379,11 +378,9 @@ load_themes (DialogData *data)
 
 	/* local themes */
 
-	style_path = gth_user_dir_get_file (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", NULL);
-	style_dir = g_file_new_for_path (style_path);
+	style_dir = gth_user_dir_get_file_for_read (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", NULL);
 	add_themes_from_dir (data, style_dir, TRUE);
 	g_object_unref (style_dir);
-	g_free (style_path);
 
 	/* system themes */
 
@@ -472,12 +469,10 @@ theme_dialog_response_cb (GtkDialog *dialog,
 	new_theme = theme->file == NULL;
 
 	if (theme->file == NULL) {
-		char  *themes_path;
 		GFile *themes_dir;
 
-		gth_user_dir_make_dir_for_file (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", "themename.cst", NULL);
-		themes_path = gth_user_dir_get_file (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", NULL);
-		themes_dir = g_file_new_for_path (themes_path);
+		gth_user_dir_mkdir_with_parents (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", NULL);
+		themes_dir = gth_user_dir_get_file_for_read (GTH_DIR_DATA, GTHUMB_DIR, "contact_sheet_themes", NULL);
 		theme->file = _g_file_create_unique (themes_dir,
 						     theme->display_name,
 						     ".cst",
@@ -488,7 +483,6 @@ theme_dialog_response_cb (GtkDialog *dialog,
 		}
 
 		g_object_unref (themes_dir);
-		g_free (themes_path);
 
 		if (theme->file == NULL)
 			return;
@@ -501,13 +495,13 @@ theme_dialog_response_cb (GtkDialog *dialog,
 		return;
 	}
 
-	if (! g_write_file (theme->file,
-			    FALSE,
-			    G_FILE_CREATE_NONE,
-			    buffer,
-			    buffer_size,
-			    NULL,
-			    &error))
+	if (! _g_file_write (theme->file,
+			     FALSE,
+			     G_FILE_CREATE_NONE,
+			     buffer,
+			     buffer_size,
+			     NULL,
+			     &error))
 	{
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), _("Could not save the theme"), error);
 		g_clear_error (&error);
@@ -735,20 +729,20 @@ dlg_contact_sheet (GthBrowser *browser,
 
 	default_mime_type = g_settings_get_string (data->settings, PREF_CONTACT_SHEET_MIME_TYPE);
 	active_index = 0;
-	savers = gth_main_get_type_set ("pixbuf-saver");
+	savers = gth_main_get_type_set ("image-saver");
 	for (i = 0; (savers != NULL) && (i < savers->len); i++) {
-		GthPixbufSaver *saver;
+		GthImageSaver *saver;
 		GtkTreeIter     iter;
 
 		saver = g_object_new (g_array_index (savers, GType, i), NULL);
 
-		if (g_str_equal (default_mime_type, gth_pixbuf_saver_get_mime_type (saver)))
+		if (g_str_equal (default_mime_type, gth_image_saver_get_mime_type (saver)))
 			active_index = i;
 
 		gtk_list_store_append (GTK_LIST_STORE (GET_WIDGET ("filetype_liststore")), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("filetype_liststore")), &iter,
-				    FILE_TYPE_COLUMN_MIME_TYPE, gth_pixbuf_saver_get_mime_type (saver),
-				    FILE_TYPE_COLUMN_DEFAULT_EXTENSION, gth_pixbuf_saver_get_default_ext (saver),
+				    FILE_TYPE_COLUMN_MIME_TYPE, gth_image_saver_get_mime_type (saver),
+				    FILE_TYPE_COLUMN_DEFAULT_EXTENSION, gth_image_saver_get_default_ext (saver),
 				    -1);
 
 		g_object_unref (saver);
