@@ -24,7 +24,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <gst/gst.h>
-#include <gst/interfaces/xoverlay.h>
+#include <gst/video/videooverlay.h>
 #include <gthumb.h>
 #include <extensions/gstreamer_utils/gstreamer-utils.h>
 #include "actions.h"
@@ -65,7 +65,7 @@ struct _GthMediaViewerPagePrivate {
 	gboolean        has_video;
 	gboolean        has_audio;
 	gulong          update_progress_id;
-	guint           set_to_pause_id;
+	gulong          update_volume_id;
 	gdouble         rate;
 	GtkWidget      *mediabar;
 	GtkWidget      *fullscreen_toolbar;
@@ -325,11 +325,11 @@ update_current_position_bar (GthMediaViewerPage *self)
         gint64    current_value = 0;
 
         format = GST_FORMAT_TIME;
-        if (gst_element_query_position (self->priv->playbin, &format, &current_value)) {
+        if (gst_element_query_position (self->priv->playbin, format, &current_value)) {
         	char *s;
 
 		if (self->priv->duration <= 0) {
-			gst_element_query_duration (self->priv->playbin, &format, &self->priv->duration);
+			gst_element_query_duration (self->priv->playbin, format, &self->priv->duration);
 			s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (self->priv->duration));
 			gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_duration")), s);
 
@@ -414,15 +414,18 @@ position_scale_button_press_event_cb (GtkWidget      *widget,
 				      GdkEventButton *event,
 				      gpointer        user_data)
 {
+#if ! GTK_CHECK_VERSION(3,6,0)
 	/* In a video player when clicking on the progress bar the user expect
 	 * to jump to the specified position directly instead of scrolling one
 	 * page up or down as happens by default in Gtk+.  The button 2
-	 * behavior is what we want by default for button 1. */
+	 * behavior is what we want by default for button 1.  This is the
+	 * default behaviour in Gtk+ 3.6.0 or greater. */
 
 	if (event->button == 1)
 		event->button = 2;
 	else if (event->button == 2)
 		event->button = 1;
+#endif
 
 	return FALSE;
 }
@@ -435,11 +438,13 @@ position_scale_button_release_event_cb (GtkWidget      *widget,
 {
 	GthMediaViewerPage *self = user_data;
 
+#if ! GTK_CHECK_VERSION(3,6,0)
 	/* Swap button 1 and 2 behaviors, see position_scale_button_press_event_cb */
 	if (event->button == 1)
 		event->button = 2;
 	else if (event->button == 2)
 		event->button = 1;
+#endif
 
 	if (self->priv->playing)
 		self->priv->block_next_jump = TRUE;
@@ -606,13 +611,18 @@ play_faster_button_clicked_cb (GtkButton *button,
 }
 
 
-static void
+static gboolean
 update_volume_from_playbin (GthMediaViewerPage *self)
 {
 	double volume;
 
+	if (self->priv->update_volume_id != 0) {
+		g_source_remove (self->priv->update_volume_id);
+		self->priv->update_volume_id = 0;
+	}
+
 	if ((self->priv->builder == NULL) || (self->priv->playbin == NULL))
-		return;
+		return FALSE;
 
 	g_object_get (self->priv->playbin, "volume", &volume, NULL);
 
@@ -632,6 +642,8 @@ update_volume_from_playbin (GthMediaViewerPage *self)
 	g_signal_handlers_block_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("volume_togglebutton")), volume == 0.0);
 	g_signal_handlers_unblock_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
+
+	return FALSE;
 }
 
 
@@ -816,7 +828,7 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 	gtk_widget_show (self->priv->area_box);
 	gth_browser_set_viewer_widget (browser, self->priv->area_box);
 
-	gtk_widget_realize (self->priv->area_box);
+	gtk_widget_realize (self->priv->area);
 
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 }
@@ -872,6 +884,11 @@ gth_media_viewer_page_real_deactivate (GthViewerPage *base)
                 self->priv->update_progress_id = 0;
         }
 
+        if (self->priv->update_volume_id != 0) {
+                g_source_remove (self->priv->update_volume_id);
+                self->priv->update_volume_id = 0;
+        }
+
 	if (self->priv->playbin != NULL) {
 		save_volume (self);
 		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
@@ -895,20 +912,15 @@ set_playbin_window (GstBus             *bus,
 		    GthMediaViewerPage *self)
 {
 	/* ignore anything but 'prepare-xwindow-id' element messages */
-
-	if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
-		return GST_BUS_PASS;
-	if (! gst_structure_has_name (message->structure, "prepare-xwindow-id"))
+	if (! gst_is_video_overlay_prepare_window_handle_message (message))
 		return GST_BUS_PASS;
 
 	if (self->priv->video_window_xid != 0) {
-		GstXOverlay *xoverlay;
+		GstVideoOverlay *video_overlay;
 
-		xoverlay = GST_X_OVERLAY (GST_MESSAGE_SRC (message));
-		gst_x_overlay_set_window_handle (xoverlay, self->priv->video_window_xid);
+		video_overlay = GST_VIDEO_OVERLAY (GST_MESSAGE_SRC (message));
+		gst_video_overlay_set_window_handle (video_overlay, self->priv->video_window_xid);
 		self->priv->xwin_assigned = TRUE;
-
-		g_object_set (xoverlay, "force-aspect-ratio", TRUE, NULL);
 	}
 	else
 		g_warning ("Should have obtained video_window_xid by now!");
@@ -936,64 +948,46 @@ reset_player_state (GthMediaViewerPage *self)
 static void
 update_stream_info (GthMediaViewerPage *self)
 {
-	GList  *streaminfo;
-	GstPad *videopad;
+	GstElement *audio_sink;
+	GstElement *video_sink;
+	GstPad     *audio_pad;
+	GstPad     *video_pad;
 
-	streaminfo = NULL;
-	videopad = NULL;
+	g_object_get (self->priv->playbin,
+		      "audio-sink", &audio_sink,
+		      "video-sink", &video_sink,
+		      NULL);
 
-	g_object_get (self->priv->playbin, "stream-info", &streaminfo, NULL);
-	streaminfo = g_list_copy (streaminfo);
-	g_list_foreach (streaminfo, (GFunc) g_object_ref, NULL);
-
-	for (/* void */ ; streaminfo; streaminfo = streaminfo->next) {
-		GObject    *info;
-		int         type;
-		GParamSpec *pspec;
-		GEnumValue *val;
-
-		info = streaminfo->data;
-		if (info == NULL)
-			continue;
-
-                type = -1;
-
-		g_object_get (info, "type", &type, NULL);
-		pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
-		val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
-
-		if (strcmp (val->value_nick, "audio") == 0) {
+	if (audio_sink != NULL) {
+		audio_pad = gst_element_get_static_pad (GST_ELEMENT (audio_sink), "sink");
+		if (audio_pad != NULL)
 			self->priv->has_audio = TRUE;
-		}
-		else if (strcmp (val->value_nick, "video") == 0) {
+	}
+
+	if (video_sink != NULL) {
+		video_pad = gst_element_get_static_pad (GST_ELEMENT (video_sink), "sink");
+		if (video_pad != NULL) {
+			GstCaps *caps;
+
 			self->priv->has_video = TRUE;
-			if (videopad == NULL)
-				g_object_get (info, "object", &videopad, NULL);
+
+			if ((caps = gst_pad_get_current_caps (video_pad)) != NULL) {
+				GstStructure *structure;
+				int           video_width;
+				int           video_height;
+
+				structure = gst_caps_get_structure (caps, 0);
+				gst_structure_get_fraction (structure, "framerate", &self->priv->video_fps_n, &self->priv->video_fps_d);
+				gst_structure_get_int (structure, "width", &video_width);
+				gst_structure_get_int (structure, "height", &video_height);
+
+				g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::width", video_width);
+				g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::height", video_height);
+
+				gst_caps_unref (caps);
+			}
 		}
 	}
-
-	if (videopad != NULL) {
-		GstCaps *caps;
-
-		if ((caps = gst_pad_get_negotiated_caps (videopad)) != NULL) {
-			GstStructure *structure;
-			int           video_width;
-			int           video_height;
-
-			structure = gst_caps_get_structure (caps, 0);
-			gst_structure_get_fraction (structure, "framerate", &self->priv->video_fps_n, &self->priv->video_fps_d);
-			gst_structure_get_int (structure, "width", &video_width);
-			gst_structure_get_int (structure, "height", &video_height);
-
-			g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::width", video_width);
-			g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::height", video_height);
-
-			gst_caps_unref (caps);
-		}
-	}
-
-	g_list_foreach (streaminfo, (GFunc) g_object_unref, NULL);
-	g_list_free (streaminfo);
 }
 
 
@@ -1039,6 +1033,11 @@ bus_message_cb (GstBus     *bus,
 		break;
 	}
 
+	case GST_MESSAGE_DURATION_CHANGED:
+		self->priv->duration = 0;
+		update_current_position_bar (self);
+		break;
+
 	case GST_MESSAGE_EOS:
 		reset_player_state (self);
 		break;
@@ -1046,13 +1045,13 @@ bus_message_cb (GstBus     *bus,
 	case GST_MESSAGE_BUFFERING: {
 		int percent = 0;
 		gst_message_parse_buffering (message, &percent);
-		/* g_print ("Buffering (%%%u percent done)", percent); */
+		gst_element_set_state(self->priv->playbin, (percent == 100) ? GST_STATE_PLAYING : GST_STATE_PAUSED);
 		break;
+	}
 
 	case GST_MESSAGE_ERROR:
 		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), self->priv->file_data, FALSE);
 		break;
-	}
 
 	default:
 		break;
@@ -1065,7 +1064,10 @@ playbin_notify_volume_cb (GObject    *playbin,
 			  GParamSpec *pspec,
 			  gpointer    user_data)
 {
-	update_volume_from_playbin ((GthMediaViewerPage *) user_data);
+	GthMediaViewerPage *self = user_data;
+
+	if (self->priv->update_volume_id == 0)
+		self->priv->update_volume_id = g_idle_add ((GSourceFunc) update_volume_from_playbin, user_data);
 }
 
 
@@ -1082,19 +1084,39 @@ create_playbin (GthMediaViewerPage *self)
 
 	settings = g_settings_new (GTHUMB_GSTREAMER_TOOLS_SCHEMA);
 	g_object_set (self->priv->playbin,
-		      "volume",
-		      (double) g_settings_get_int (settings, PREF_GSTREAMER_TOOLS_VOLUME) / 100.0,
+		      "volume", (double) g_settings_get_int (settings, PREF_GSTREAMER_TOOLS_VOLUME) / 100.0,
+		      "force-aspect-ratio", TRUE,
 		      NULL);
 	g_object_unref (settings);
 
-	g_signal_connect (self->priv->playbin, "notify::volume", G_CALLBACK (playbin_notify_volume_cb), self);
-
 	bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
 	gst_bus_enable_sync_message_emission (bus);
-	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) set_playbin_window, self);
+	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) set_playbin_window, self, NULL);
 	gst_bus_add_signal_watch (bus);
-	g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
 
+	g_signal_connect (self->priv->playbin,
+			  "notify::volume",
+			  G_CALLBACK (playbin_notify_volume_cb),
+			  self);
+	g_signal_connect (bus,
+			  "message",
+			  G_CALLBACK (bus_message_cb),
+			  self);
+}
+
+
+static void
+_gth_media_viewer_page_set_uri (GthMediaViewerPage *self,
+				const char         *uri,
+				GstState            state)
+{
+	g_return_if_fail (self->priv->playbin != NULL);
+
+	gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+
+	g_object_set (G_OBJECT (self->priv->playbin), "uri", uri, NULL);
+	gst_element_set_state (self->priv->playbin, state);
+	gst_element_get_state (self->priv->playbin, NULL, NULL, GST_CLOCK_TIME_NONE);
 }
 
 
@@ -1120,8 +1142,7 @@ gth_media_viewer_page_real_show (GthViewerPage *base)
 		char *uri;
 
 		uri = g_file_get_uri (self->priv->file_data->file);
-		g_object_set (G_OBJECT (self->priv->playbin), "uri", uri, NULL);
-		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+		_gth_media_viewer_page_set_uri (self, uri, GST_STATE_PLAYING);
 
 		g_free (uri);
 	}
@@ -1157,19 +1178,6 @@ gth_media_viewer_page_real_can_view (GthViewerPage *base,
 }
 
 
-static gboolean
-set_to_paused (gpointer user_data)
-{
-	GthMediaViewerPage *self = user_data;
-
-	self->priv->set_to_pause_id = 0;
-	if (self->priv->playbin != NULL)
-		gst_element_set_state (self->priv->playbin, self->priv->visible ? GST_STATE_PLAYING : GST_STATE_READY);
-
-	return FALSE;
-}
-
-
 static void
 gth_media_viewer_page_real_view (GthViewerPage *base,
 				 GthFileData   *file_data)
@@ -1217,13 +1225,8 @@ gth_media_viewer_page_real_view (GthViewerPage *base,
 	if (self->priv->playbin == NULL)
 		return;
 
-	gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
 	uri = g_file_get_uri (self->priv->file_data->file);
-	g_object_set (G_OBJECT (self->priv->playbin), "uri", uri, NULL);
-
-	if (self->priv->set_to_pause_id != 0)
-		g_source_remove (self->priv->set_to_pause_id);
-	self->priv->set_to_pause_id = gdk_threads_add_idle (set_to_paused, self);
+	_gth_media_viewer_page_set_uri (self, uri, self->priv->visible ? GST_STATE_PLAYING : GST_STATE_PAUSED);
 
 	g_free (uri);
 }
@@ -1398,16 +1401,16 @@ gth_media_viewer_page_finalize (GObject *obj)
 
 	self = GTH_MEDIA_VIEWER_PAGE (obj);
 
-	if (self->priv->set_to_pause_id != 0) {
-		g_source_remove (self->priv->set_to_pause_id);
-		self->priv->set_to_pause_id = 0;
-	}
-
         if (self->priv->update_progress_id != 0) {
                 g_source_remove (self->priv->update_progress_id);
                 self->priv->update_progress_id = 0;
         }
 
+        if (self->priv->update_volume_id != 0) {
+                g_source_remove (self->priv->update_volume_id);
+                self->priv->update_volume_id = 0;
+        }
+
 	if (self->priv->playbin != NULL) {
 		save_volume (self);
 		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
@@ -1461,7 +1464,7 @@ gth_media_viewer_page_init (GthMediaViewerPage *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_MEDIA_VIEWER_PAGE, GthMediaViewerPagePrivate);
 	self->priv->update_progress_id = 0;
-	self->priv->set_to_pause_id = 0;
+	self->priv->update_volume_id = 0;
 	self->priv->video_window_xid = 0;
 	self->priv->xwin_assigned = FALSE;
 	self->priv->has_video = FALSE;
