@@ -32,8 +32,11 @@ enum {
 static guint gth_viewer_page_signals[LAST_SIGNAL] = { 0 };
 
 
+G_DEFINE_INTERFACE (GthViewerPage, gth_viewer_page, 0)
+
+
 static void
-gth_viewer_page_base_init (gpointer g_iface)
+gth_viewer_page_default_init (GthViewerPageInterface *iface)
 {
 	static gboolean initialized = FALSE;
 
@@ -44,7 +47,7 @@ gth_viewer_page_base_init (gpointer g_iface)
 	            g_signal_new ("file-loaded",
 	        		  GTH_TYPE_VIEWER_PAGE,
 	                          G_SIGNAL_RUN_LAST,
-	                          G_STRUCT_OFFSET (GthViewerPageIface, file_loaded),
+	                          G_STRUCT_OFFSET (GthViewerPageInterface, file_loaded),
 	                          NULL, NULL,
 	                          gth_marshal_VOID__OBJECT_BOOLEAN,
 	                          G_TYPE_NONE,
@@ -56,28 +59,6 @@ gth_viewer_page_base_init (gpointer g_iface)
 }
 
 
-GType
-gth_viewer_page_get_type (void) {
-	static GType gth_viewer_page_type_id = 0;
-	if (gth_viewer_page_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (GthViewerPageIface),
-			(GBaseInitFunc) gth_viewer_page_base_init,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) NULL,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			0,
-			0,
-			(GInstanceInitFunc) NULL,
-			NULL
-		};
-		gth_viewer_page_type_id = g_type_register_static (G_TYPE_INTERFACE, "GthViewerPage", &g_define_type_info, 0);
-	}
-	return gth_viewer_page_type_id;
-}
-
-
 void
 gth_viewer_page_activate (GthViewerPage *self,
 			  GthBrowser    *browser)
@@ -192,6 +173,34 @@ gth_viewer_page_revert (GthViewerPage *self)
 }
 
 
+void
+gth_viewer_page_update_info (GthViewerPage *self,
+		  	     GthFileData   *file_data)
+{
+	GTH_VIEWER_PAGE_GET_INTERFACE (self)->update_info (self, file_data);
+}
+
+
+void
+gth_viewer_page_show_properties (GthViewerPage *self,
+				 gboolean       show)
+{
+	if (GTH_VIEWER_PAGE_GET_INTERFACE (self)->show_properties != NULL)
+		GTH_VIEWER_PAGE_GET_INTERFACE (self)->show_properties (self, show);
+}
+
+
+void
+gth_viewer_page_shrink_wrap (GthViewerPage *self,
+			     gboolean       value,
+			     int           *other_width,
+			     int           *other_height)
+{
+	if (GTH_VIEWER_PAGE_GET_INTERFACE (self)->shrink_wrap != NULL)
+		GTH_VIEWER_PAGE_GET_INTERFACE (self)->shrink_wrap (self, value, other_width, other_height);
+}
+
+
 void
 gth_viewer_page_file_loaded (GthViewerPage *self,
 			     GthFileData   *file_data,
